import type { ExtractPropTypes, PropType, VNode } from 'vue-demi';
import type { AutoPlacementOptions, FlipOptions, OffsetOptions, ShiftOptions } from '@floating-ui/core';
import type { FloatingType, FloatingData, ReferenceType, VirtualElement } from '@visoning/vue-floating-core';
export interface PopupExposed {
    updatePosition: () => void;
    getFloatingData: () => FloatingData | undefined;
    getElements: () => {
        floating: FloatingType | undefined;
        reference: ReferenceType | undefined;
    };
}
export declare type PopupSlotProps = FloatingData;
export declare const FloatingCreatorListenerPropsForwarder: import("@visoning/vue-utility").UseListenerForwardReturn<import("@visoning/vue-utility").WithDefaultProps<{
    readonly reference: {
        type: PropType<ReferenceType>;
    };
    readonly floating: {
        type: PropType<HTMLElement>;
    };
    readonly disabled: BooleanConstructor;
    readonly placement: PropType<import("@floating-ui/core").Placement>;
    readonly strategy: PropType<import("@floating-ui/core").Strategy>;
    readonly middleware: PropType<import("@floating-ui/core").Middleware[]>;
    readonly autoUpdate: PropType<boolean | import("@visoning/vue-floating-core").UseAutoUpdateOptions>;
    readonly onFloatingDataUpdate: PropType<(data: {
        x: number;
        y: number;
        strategy: import("@floating-ui/core").Strategy;
        placement: import("@floating-ui/core").Placement;
        middlewareData: {
            [x: string]: any;
            arrow?: {
                x?: number;
                y?: number;
                centerOffset: number;
            };
            autoPlacement?: {
                index?: number;
                overflows: {
                    placement: import("@floating-ui/core").Placement;
                    overflows: number[];
                }[];
            };
            flip?: {
                index?: number;
                overflows: {
                    placement: import("@floating-ui/core").Placement;
                    overflows: number[];
                }[];
            };
            hide?: {
                referenceHidden?: boolean;
                escaped?: boolean;
                referenceHiddenOffsets?: {
                    left: number;
                    top: number;
                    bottom: number;
                    right: number;
                };
                escapedOffsets?: {
                    left: number;
                    top: number;
                    bottom: number;
                    right: number;
                };
            };
            offset?: {
                x: number;
                y: number;
            };
            shift?: {
                x: number;
                y: number;
            };
        };
    }) => void>;
}, {
    readonly placement: "bottom";
    readonly strategy: "absolute";
    readonly autoUpdate: true;
}>, "onFloatingDataUpdate">;
export declare const PopupPropsType: {
    /**
     * Whether to open popup.
     */
    readonly open: BooleanConstructor;
    /**
     * Whether to disable the floating.
     */
    readonly disabled: BooleanConstructor;
    /**
     * You can position a floating element relative to a virtual element instead of a real one.
     * This enables things like positioning context menus or following the cursor.
     * @see https://floating-ui.com/docs/virtual-elements
     */
    readonly virtualElement: PropType<VirtualElement>;
    /**
     * When this value is `true`, popup will be teleport to the target.
     * The teleport to target must be already in the DOM when the <Teleport> component is mounted.
     * @see https://vuejs.org/guide/built-ins/teleport.html
     *
     * Or you can customize the teleport logic, set `appendTo` to false, and use `floatingWrapper`,
     * for simple example
     * ```tsx
     * {
     *   appendTo: false,
     *   floatingWrapper: (floating) => h(
     *    Teleport,
     *    {
     *      to: anyTarget
     *    },
     *    {
     *      default: () => [floating]
     *    }
     *   )
     * }
     * ```
     * @default 'body'
     */
    readonly appendTo: PropType<string | false | HTMLElement>;
    /**
     * When this value is `false`, even if the popup is closed, floating will update the position as needed,
     * unless popup is disabled.
     */
    readonly autoUpdateOnClosed: BooleanConstructor;
    /**
     * Displaces the popup element from its core placement along the specified axes.
     * @see https://floating-ui.com/docs/offset
     */
    readonly offset: PropType<OffsetOptions>;
    /**
     * Moves the popup element along the specified axes in order to keep it in view.
     * @default true
     * @see https://floating-ui.com/docs/shift
     */
    readonly shift: PropType<boolean | ShiftOptions>;
    /**
     * Changes the placement of the popup element to the opposite one by default in order to keep it in view.
     * @see https://floating-ui.com/docs/flip
     */
    readonly flip: PropType<boolean | FlipOptions>;
    /**
     * Chooses the placement that has the most space available automatically.
     *
     * You’ll want to use this instead of `flip` if you don't want to give the popup element a “preferred” placement
     * and let it choose a placement for you.
     * @see https://floating-ui.com/docs/autoPlacement
     */
    readonly autoPlacement: PropType<boolean | AutoPlacementOptions>;
    /**
     * This determines whether GPU-accelerated styles are used to position the popup.
     * When this value is `true`, popup will use the 3D transforms.
     * When this value is `false`, popup will use top/left properties.
     */
    readonly gpuAcceleration: BooleanConstructor;
    /**
     * Whether destroy popup when closed.
     */
    readonly destoryedOnClosed: BooleanConstructor;
    /**
     * HTML attributes of node.
     */
    readonly referenceProps: PropType<Record<string, any>>;
    /**
     * HTML attributes of node.
     */
    readonly popupProps: PropType<Record<string, any>>;
    /**
     * Custom popup node wrapper.
     * The basic DOM structure is `<Floating/> -> <Popup/> -> <PopupContent/>`.
     */
    readonly popupWrapper: PropType<(popup: VNode | null) => VNode>;
    /**
     * Custom floating node wrapper.
     * The basic DOM structure is `<Floating/> -> <Popup/> -> <PopupContent/>`.
     */
    readonly floatingWrapper: PropType<(popup: VNode | null) => VNode>;
    /**
     * Popup z-index.
     */
    readonly zIndex: NumberConstructor;
    readonly onFloatingDataUpdate: PropType<(data: {
        x: number;
        y: number;
        strategy: import("@floating-ui/core").Strategy;
        placement: import("@floating-ui/core").Placement;
        middlewareData: {
            [x: string]: any;
            arrow?: {
                x?: number;
                y?: number;
                centerOffset: number;
            };
            autoPlacement?: {
                index?: number;
                overflows: {
                    placement: import("@floating-ui/core").Placement;
                    overflows: number[];
                }[];
            };
            flip?: {
                index?: number;
                overflows: {
                    placement: import("@floating-ui/core").Placement;
                    overflows: number[];
                }[];
            };
            hide?: {
                referenceHidden?: boolean;
                escaped?: boolean;
                referenceHiddenOffsets?: {
                    left: number;
                    top: number;
                    bottom: number;
                    right: number;
                };
                escapedOffsets?: {
                    left: number;
                    top: number;
                    bottom: number;
                    right: number;
                };
            };
            offset?: {
                x: number;
                y: number;
            };
            shift?: {
                x: number;
                y: number;
            };
        };
    }) => void>;
    readonly reference: {
        type: PropType<ReferenceType>;
    };
    readonly floating: {
        type: PropType<HTMLElement>;
    };
    readonly strategy: Omit<{
        type: new (...args: any[]) => import("@floating-ui/core").Strategy & {};
    } | {
        type: () => import("@floating-ui/core").Strategy;
    } | {
        type: ((new (...args: any[]) => import("@floating-ui/core").Strategy & {}) | (() => import("@floating-ui/core").Strategy))[];
    }, "default"> & {
        default: "absolute";
    };
    readonly middleware: PropType<import("@floating-ui/core").Middleware[]>;
    readonly placement: Omit<{
        type: new (...args: any[]) => import("@floating-ui/core").Placement & {};
    } | {
        type: () => import("@floating-ui/core").Placement;
    } | {
        type: ((new (...args: any[]) => import("@floating-ui/core").Placement & {}) | (() => import("@floating-ui/core").Placement))[];
    }, "default"> & {
        default: "bottom";
    };
    readonly autoUpdate: Omit<{
        type: new (...args: any[]) => (boolean | import("@visoning/vue-floating-core").UseAutoUpdateOptions) & {};
    } | {
        type: () => boolean | import("@visoning/vue-floating-core").UseAutoUpdateOptions;
    } | {
        type: ((new (...args: any[]) => (boolean | import("@visoning/vue-floating-core").UseAutoUpdateOptions) & {}) | (() => boolean | import("@visoning/vue-floating-core").UseAutoUpdateOptions))[];
    }, "default"> & {
        default: true;
    };
};
export declare const PopupDefaultProps: {
    readonly appendTo: "body";
    readonly shift: true;
    readonly placement: "bottom";
    readonly strategy: "absolute";
    readonly autoUpdate: true;
};
export declare const PopupProps: import("@visoning/vue-utility").WithDefaultProps<{
    /**
     * Whether to open popup.
     */
    readonly open: BooleanConstructor;
    /**
     * Whether to disable the floating.
     */
    readonly disabled: BooleanConstructor;
    /**
     * You can position a floating element relative to a virtual element instead of a real one.
     * This enables things like positioning context menus or following the cursor.
     * @see https://floating-ui.com/docs/virtual-elements
     */
    readonly virtualElement: PropType<VirtualElement>;
    /**
     * When this value is `true`, popup will be teleport to the target.
     * The teleport to target must be already in the DOM when the <Teleport> component is mounted.
     * @see https://vuejs.org/guide/built-ins/teleport.html
     *
     * Or you can customize the teleport logic, set `appendTo` to false, and use `floatingWrapper`,
     * for simple example
     * ```tsx
     * {
     *   appendTo: false,
     *   floatingWrapper: (floating) => h(
     *    Teleport,
     *    {
     *      to: anyTarget
     *    },
     *    {
     *      default: () => [floating]
     *    }
     *   )
     * }
     * ```
     * @default 'body'
     */
    readonly appendTo: PropType<string | false | HTMLElement>;
    /**
     * When this value is `false`, even if the popup is closed, floating will update the position as needed,
     * unless popup is disabled.
     */
    readonly autoUpdateOnClosed: BooleanConstructor;
    /**
     * Displaces the popup element from its core placement along the specified axes.
     * @see https://floating-ui.com/docs/offset
     */
    readonly offset: PropType<OffsetOptions>;
    /**
     * Moves the popup element along the specified axes in order to keep it in view.
     * @default true
     * @see https://floating-ui.com/docs/shift
     */
    readonly shift: PropType<boolean | ShiftOptions>;
    /**
     * Changes the placement of the popup element to the opposite one by default in order to keep it in view.
     * @see https://floating-ui.com/docs/flip
     */
    readonly flip: PropType<boolean | FlipOptions>;
    /**
     * Chooses the placement that has the most space available automatically.
     *
     * You’ll want to use this instead of `flip` if you don't want to give the popup element a “preferred” placement
     * and let it choose a placement for you.
     * @see https://floating-ui.com/docs/autoPlacement
     */
    readonly autoPlacement: PropType<boolean | AutoPlacementOptions>;
    /**
     * This determines whether GPU-accelerated styles are used to position the popup.
     * When this value is `true`, popup will use the 3D transforms.
     * When this value is `false`, popup will use top/left properties.
     */
    readonly gpuAcceleration: BooleanConstructor;
    /**
     * Whether destroy popup when closed.
     */
    readonly destoryedOnClosed: BooleanConstructor;
    /**
     * HTML attributes of node.
     */
    readonly referenceProps: PropType<Record<string, any>>;
    /**
     * HTML attributes of node.
     */
    readonly popupProps: PropType<Record<string, any>>;
    /**
     * Custom popup node wrapper.
     * The basic DOM structure is `<Floating/> -> <Popup/> -> <PopupContent/>`.
     */
    readonly popupWrapper: PropType<(popup: VNode | null) => VNode>;
    /**
     * Custom floating node wrapper.
     * The basic DOM structure is `<Floating/> -> <Popup/> -> <PopupContent/>`.
     */
    readonly floatingWrapper: PropType<(popup: VNode | null) => VNode>;
    /**
     * Popup z-index.
     */
    readonly zIndex: NumberConstructor;
    readonly onFloatingDataUpdate: PropType<(data: {
        x: number;
        y: number;
        strategy: import("@floating-ui/core").Strategy;
        placement: import("@floating-ui/core").Placement;
        middlewareData: {
            [x: string]: any;
            arrow?: {
                x?: number;
                y?: number;
                centerOffset: number;
            };
            autoPlacement?: {
                index?: number;
                overflows: {
                    placement: import("@floating-ui/core").Placement;
                    overflows: number[];
                }[];
            };
            flip?: {
                index?: number;
                overflows: {
                    placement: import("@floating-ui/core").Placement;
                    overflows: number[];
                }[];
            };
            hide?: {
                referenceHidden?: boolean;
                escaped?: boolean;
                referenceHiddenOffsets?: {
                    left: number;
                    top: number;
                    bottom: number;
                    right: number;
                };
                escapedOffsets?: {
                    left: number;
                    top: number;
                    bottom: number;
                    right: number;
                };
            };
            offset?: {
                x: number;
                y: number;
            };
            shift?: {
                x: number;
                y: number;
            };
        };
    }) => void>;
    readonly reference: {
        type: PropType<ReferenceType>;
    };
    readonly floating: {
        type: PropType<HTMLElement>;
    };
    readonly strategy: Omit<{
        type: new (...args: any[]) => import("@floating-ui/core").Strategy & {};
    } | {
        type: () => import("@floating-ui/core").Strategy;
    } | {
        type: ((new (...args: any[]) => import("@floating-ui/core").Strategy & {}) | (() => import("@floating-ui/core").Strategy))[];
    }, "default"> & {
        default: "absolute";
    };
    readonly middleware: PropType<import("@floating-ui/core").Middleware[]>;
    readonly placement: Omit<{
        type: new (...args: any[]) => import("@floating-ui/core").Placement & {};
    } | {
        type: () => import("@floating-ui/core").Placement;
    } | {
        type: ((new (...args: any[]) => import("@floating-ui/core").Placement & {}) | (() => import("@floating-ui/core").Placement))[];
    }, "default"> & {
        default: "bottom";
    };
    readonly autoUpdate: Omit<{
        type: new (...args: any[]) => (boolean | import("@visoning/vue-floating-core").UseAutoUpdateOptions) & {};
    } | {
        type: () => boolean | import("@visoning/vue-floating-core").UseAutoUpdateOptions;
    } | {
        type: ((new (...args: any[]) => (boolean | import("@visoning/vue-floating-core").UseAutoUpdateOptions) & {}) | (() => boolean | import("@visoning/vue-floating-core").UseAutoUpdateOptions))[];
    }, "default"> & {
        default: true;
    };
}, {
    readonly appendTo: "body";
    readonly shift: true;
    readonly placement: "bottom";
    readonly strategy: "absolute";
    readonly autoUpdate: true;
}>;
export declare type PopupProps = ExtractPropTypes<typeof PopupProps>;
